/*
 ******************************************************************************

 LIBGPF: libggi io target pre/post conversion functions

 Copyright (C) 2001 Christoph Egger     [Christoph_Egger@t-online.de]

 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
 to deal in the Software without restriction, including without limitation
 the rights to use, copy, modify, merge, publish, distribute, sublicense,
 and/or sell copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 ******************************************************************************
 */

#include "libggi_gpf.h"


ssize_t GPF_LIBGGI_transfernextstreamblock_input(gpf_handle_t handle,
					struct gpf_pipeline_item_t *pipe)
{
	ssize_t rc;
	struct gpf_pipeline_t *pipeline = NULL;
	struct gpf_handle_opio_t *opio = NULL;
	struct libggigpf_priv *priv = NULL;

	LIB_ASSERT(handle != NULL);
	LIB_ASSERT(pipe != NULL);

	GPFDPRINT_TARGET("%s:%s:%i: handle: %p, pipe: %p\n",
			DEBUG_INFO, handle, pipe);

	pipeline = handle->pipeline;
	opio = handle->opio_input;
	priv = LIBGGI_PRIV(opio);

	LIB_ASSERT(PIPELINE_READBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_READBUFSIZE(pipeline) > 0);
	LIB_ASSERT(PIPELINE_WRITEBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_WRITEBUFSIZE(pipeline) > 0);

	LIB_ASSERT(pipeline->buf_conv == NULL);
	LIB_ASSERT(pipeline->buf_conv_size == 0);
	pipeline->buf_conv = PIPELINE_READBUF(pipeline);
	pipeline->buf_conv_size = PIPELINE_READBUFSIZE(pipeline);

	rc = opio->read(opio,
			PIPELINE_READBUF(pipeline),
			PIPELINE_READBUFSIZE(pipeline));
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	rc = pipeline->write(pipeline, rc, 1 /* realloc */);
	fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

	pipeline->buf_conv = NULL;
	pipeline->buf_conv_size = 0;

	return rc;
}	/* GPF_LIBGGI_transfernextstreamblock_input */



ssize_t GPF_LIBGGI_transfernextstreamblock_output(gpf_handle_t handle,
					struct gpf_pipeline_item_t *pipe)
{
	ssize_t sum;
	ssize_t rc;
	struct gpf_handle_opio_t *opio = NULL;
	struct gpf_pipeline_t *pipeline = NULL;
	struct libggigpf_priv *priv = NULL;

	LIB_ASSERT(handle != NULL);
	LIB_ASSERT(pipe != NULL);

	GPFDPRINT_TARGET("%s:%s:%i: handle: %p, pipe: %p\n",
			DEBUG_INFO, handle, pipe);

	pipeline = handle->pipeline;
	opio = handle->opio_output;
	priv = LIBGGI_PRIV(opio);

	LIB_ASSERT(PIPELINE_READBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_READBUFSIZE(pipeline) > 0);
	LIB_ASSERT(PIPELINE_WRITEBUF(pipeline) != NULL);
	LIB_ASSERT(PIPELINE_WRITEBUFSIZE(pipeline) > 0);

	if (pipeline->buf_conv != NULL) {
		/* happens, when it was used
		 * by a pipe sublib 
		 */
		free(pipeline->buf_conv);
		pipeline->buf_conv_size = 0;
	}	/* if */

	if (PIPELINE_WRITEBUFSIZE(pipeline)
	   != PIPELINE_READBUFSIZE(pipeline))
	{
		void *buf_tmp;
		buf_tmp = realloc(PIPELINE_WRITEBUF(pipeline),
				PIPELINE_READBUFSIZE(pipeline));
		if (!buf_tmp) {
			goto norealloc;
		}	/* if */
		PIPELINE_WRITEBUF(pipeline) = buf_tmp;
		PIPELINE_WRITEBUFSIZE(pipeline) = PIPELINE_READBUFSIZE(pipeline);
	}	/* if */

norealloc:
	pipeline->buf_conv = PIPELINE_WRITEBUF(pipeline);
	pipeline->buf_conv_size = PIPELINE_WRITEBUFSIZE(pipeline);

	sum = 0;
	do {
		rc = pipeline->read(pipeline,
			pipeline->buf_conv_size, 1 /* cache */);
		fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);

		if (rc == 0) break;

		rc = opio->write(opio,
			PIPELINE_WRITEBUF(pipeline),
			PIPELINE_WRITEBUFSIZE(pipeline));
		fprintf(stderr, "%s:%s:%i: rc: %i\n", DEBUG_INFO, rc);
		sum += rc;
	} while (rc > 0);

	pipeline->buf_conv = NULL;
	pipeline->buf_conv_size = 0;

	return sum;
}	/* GPF_LIBGGI_transfernextstreamblock_output */
